﻿using System;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Text;
using System.IO;
using System.Linq;
namespace MuseProxyHMAC
{
	public class MuseProxyHMAC
	{
		public String generatedURL = null;

		public String generatedHmacDigest = null;

		public String museProxyURL = null;

		public String applicationID = null;

		public String secret = null;

		public String algorithm = null;

		public String separator = null;

		public MuseProxyHMAC (String museProxyURL, String applicationID, String secret, String algorithm, String separator, Dictionary<String, String> usedParameters)
		{
			this.museProxyURL = museProxyURL;
			this.applicationID = applicationID;
			this.secret = secret;
			this.algorithm = algorithm;
			this.separator = separator;

			String value = getValueForGenerateDigest (usedParameters ,separator);
			String parametersToSend = getParametersToCreateURL (usedParameters);
			this.generatedHmacDigest = generateHmacDigest (algorithm,secret,value);
			this.generatedURL = getHmacURL (museProxyURL, applicationID, generatedHmacDigest, parametersToSend);
		}

		public String getHmacURL(String museProxyURL, String applicationID, String generatedHmacDigest, String parametersToSend) {
			return museProxyURL + "/" + applicationID + "?sig=" + generatedHmacDigest + parametersToSend;
		}


		public String getParametersToCreateURL(Dictionary<string, string> usedParameters){
			String parametersToSend = "";
			foreach( KeyValuePair<string, string> param in usedParameters )
			{
				String key = param.Key;
				if (!key.Equals ("userAddress") && !key.Equals ("userAgent") && !key.Equals ("referer")) {
					parametersToSend += "&" + key + "=" + param.Value;
				}
			}
			return parametersToSend;
		}

		public String getValueForGenerateDigest(Dictionary<String, String> usedParameters, String separator){
			String value = "";
			int length = usedParameters.Count;
			for (int i = 0; i < length; i++) {
				if (i < length - 1) {
					value += usedParameters.Values.ElementAt(i) + separator;
				} else {
					value += usedParameters.Values.ElementAt(i); 
				}
			}
			return value;
		}

		public String generateHmacDigest(String algorithm, String secret, String value){
			byte[] key = System.Text.ASCIIEncoding.Default.GetBytes(secret);
			byte[] byteArray = Encoding.ASCII.GetBytes(value);
			MemoryStream stream = new MemoryStream(byteArray);
			String digest = null;
			if (algorithm.Equals ("sha1")) {
				HMACSHA1 hmacSHA1 = new HMACSHA1 (key);
				digest = hmacSHA1.ComputeHash (stream).Aggregate ("", (s, e) => s + String.Format ("{0:x2}", e), s => s);
			} else if (algorithm.Equals ("md5")) {
				HMACMD5 hmacMD5 = new HMACMD5 (key);
				digest = hmacMD5.ComputeHash (stream).Aggregate ("", (s, e) => s + String.Format ("{0:x2}", e), s => s);
			} else if (algorithm.Equals ("sha256")) {
				HMACSHA256 hmacSHA256 = new HMACSHA256 (key);
				digest = hmacSHA256.ComputeHash (stream).Aggregate ("", (s, e) => s + String.Format ("{0:x2}", e), s => s);
			} else if (algorithm.Equals ("sha384")) {
				HMACSHA384 hmacSHA384 = new HMACSHA384 (key);
				digest = hmacSHA384.ComputeHash (stream).Aggregate ("", (s, e) => s + String.Format ("{0:x2}", e), s => s);
			} else if(algorithm.Equals ("sha512")){
				HMACSHA512 hmacSHA512 = new HMACSHA512 (key);
				digest = hmacSHA512.ComputeHash (stream).Aggregate ("", (s, e) => s + String.Format ("{0:x2}", e), s => s);
			}
			return digest;
		}
	}
}

